package edu.washington.cs.knowitall.extractor;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Stack;

import org.apache.commons.lang.StringUtils;

import edu.washington.cs.knowitall.nlp.ChunkedSentence;
import edu.washington.cs.knowitall.nlp.ChunkedSentenceReader;
import edu.washington.cs.knowitall.nlp.NpList;
import edu.washington.cs.knowitall.util.DefaultObjects;
import edu.washington.cs.knowitall.util.Range;

public class NpListExtractor extends Extractor<ChunkedSentence, NpList> {

    private int minNumNps = 3;

    public NpListExtractor() {
    }

    public NpListExtractor(int minNumNps) {
        this.minNumNps = minNumNps;
    }

    public int getMinNumNps() {
        return minNumNps;
    }

    public void setMinNumNps(int val) {
        this.minNumNps = val;
    }

    private boolean separatedByComma(Range r1, Range r2, ChunkedSentence sent) {
        int last = r1.getLastIndex();
        int first = r2.getStart();
        String bet = sent.getTokens().get(last + 1);
        return last == first - 2 && bet.equals(",");
    }

    private boolean isAndOr(String tok) {
        return tok.equals("and") || tok.equals("or");
    }

    private boolean separatedByAndOr(Range r1, Range r2, ChunkedSentence sent) {
        int last = r1.getLastIndex();
        int first = r2.getStart();
        if (last == first - 2 && isAndOr(sent.getTokens().get(last + 1))) {
            return true;
        } else if (last == first - 3 && sent.getTokens().get(last+1).equals(",") && isAndOr(sent.getTokens().get(last+2))) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public Iterable<NpList> extractCandidates(ChunkedSentence sent) {
        ArrayList<NpList> results = new ArrayList<NpList>();
        Stack<Range> ranges = new Stack<Range>();
        for (Range cur : sent.getNpChunkRanges()) {
            if (ranges.size() == 0) {
                ranges.add(cur);
            } else {
                Range prev = ranges.lastElement();
                if (separatedByComma(prev, cur, sent)) {
                    ranges.add(cur);
                } else if (separatedByAndOr(prev, cur, sent)) {
                    ranges.add(cur);
                    if (ranges.size() >= minNumNps) {
                        NpList npl = new NpList(sent, ranges);
                        results.add(npl);
                    }
                    ranges.clear();
                } else {
                    ranges.clear();
                    ranges.add(cur);
                }
            }
        }
        return results;
    }

    public static void main(String[] args) throws Exception {
        BufferedReader reader;
        if (args.length == 0) {
            reader = new BufferedReader(new InputStreamReader(System.in));
        } else {
            reader = new BufferedReader(new FileReader(args[0]));
        }
        ChunkedSentenceReader sentReader = DefaultObjects.getDefaultSentenceReaderHtml(reader);
        NpListExtractor extractor = new NpListExtractor();
        for (ChunkedSentence sent : sentReader) {
            for (NpList npList : extractor.extractCandidates(sent)) {
                System.out.println(StringUtils.join(npList.getNpStrings().iterator(), "\t"));
            }
        }
    }

}
