package edu.washington.cs.knowitall.extractor;


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import edu.washington.cs.knowitall.extractor.mapper.OrdinalPreprocessor;
import edu.washington.cs.knowitall.nlp.ChunkedSentence;
import edu.washington.cs.knowitall.nlp.ChunkedSentenceReader;
import edu.washington.cs.knowitall.nlp.OrdinalNumber;
import edu.washington.cs.knowitall.nlp.OrdinalUtils;
import edu.washington.cs.knowitall.nlp.extraction.OrdinalPhraseExtraction;
import edu.washington.cs.knowitall.util.DefaultObjects;
import edu.washington.cs.knowitall.util.Range;

public class OrdinalPhraseExtractor extends Extractor<ChunkedSentence, OrdinalPhraseExtraction> {

    private static boolean isSuperlative(String s) {
        return s.equals("JJS") || s.equals("RBS");
    }

    private static OrdinalNumber getOrdinalNumber(ChunkedSentence sent, Range r) {
        List<Range> ordinalRanges = OrdinalUtils.getOrdinalRanges(sent.getTokens(r).toArray(new String[] {}));
        if (ordinalRanges.size() == 1) {
            Range ordRange = ordinalRanges.get(0).shift(r.getStart());
            int val = OrdinalUtils.parseOrdinal(sent.getTokensAsString(ordRange));
            if (val > 0) {
                return new OrdinalNumber(sent, ordRange, val);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    private static boolean hasSuperlative(ChunkedSentence sent, Range r) {
        for (int i : r) {
            if (isSuperlative(sent.getPosTags().get(i))) {
                return true;
            }
        }
        return false;
    }

    @Override
    public Collection<OrdinalPhraseExtraction> extractCandidates(ChunkedSentence sent) {
        List<OrdinalPhraseExtraction> extrs = new ArrayList<OrdinalPhraseExtraction>();
        for (Range npRange : sent.getNpChunkRanges()) {
            OrdinalNumber ordinal = getOrdinalNumber(sent, npRange);
            if (ordinal != null || hasSuperlative(sent, npRange)) {
                OrdinalPhraseExtraction extr = new OrdinalPhraseExtraction(sent, npRange, ordinal);
                extrs.add(extr);
            }
        }
        return extrs;
    }

    public static void main(String[] args) throws Exception {
        BufferedReader reader;
        if (args.length == 0) {
            reader = new BufferedReader(new InputStreamReader(System.in));
        } else {
            reader = new BufferedReader(new FileReader(args[0]));
        }

        OrdinalPhraseExtractor extractor = new OrdinalPhraseExtractor();
        ChunkedSentenceReader sentReader = DefaultObjects.getDefaultSentenceReaderHtml(reader);
        sentReader.getSentenceExtractor().addMapper(new OrdinalPreprocessor());

        for (ChunkedSentence sentence : sentReader.getSentences()) {
            Iterable<OrdinalPhraseExtraction> extrs = extractor.extract(sentence);
            for (OrdinalPhraseExtraction extr : extrs) {
                System.out.println("\t" + extr);
            }
            System.out.println();
        }
    }

}
