package edu.washington.cs.knowitall.nlp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.google.common.base.Predicate;
import com.google.common.collect.AbstractIterator;

/**
 * A class that combines an <code>Iterator<String></code> object over sentences with a
 * <code>SentenceChunker</code> object to produce an iterator over <code>NpChunkedSentence</code>
 * objects.
 * @author afader
 *
 */
public class ChunkedSentenceIterator extends AbstractIterator<ChunkedSentence> {

    private SentenceChunker chunker;
    private Iterator<String> sentIter;
    private List<Predicate<ChunkedSentence>> filters;

    /**
     * @param sentIter an iterator over <code>String</code> sentences.
     * @param chunker
     */
    public ChunkedSentenceIterator(Iterator<String> sentIter, SentenceChunker chunker) {
        this.sentIter = sentIter;
        this.chunker = chunker;
        this.filters = new ArrayList<Predicate<ChunkedSentence>>();
    }
    
    /***
     * If a chunked sentence is filtered, it will not be returned.
     * @param filter A predicate to test chunked sentences against.
     */
    public void addFilter(Predicate<ChunkedSentence> filter) {
    	filters.add(filter);
    }

    @Override
    protected ChunkedSentence computeNext() {
        if (sentIter.hasNext()) {
        	// chunk the sentence
            ChunkedSentence chunkedSentence = chunker.chunkSentence(sentIter.next());
            
            // if it fails a filter, try again
            for (Predicate<ChunkedSentence> filter : this.filters) {
            	if (filter.apply(chunkedSentence)) {
            		return computeNext();
            	}
            }
            
            // no filter hit
            return chunkedSentence;
        } else {
            return endOfData();
        }
    }
}
