package edu.washington.cs.knowitall.nlp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import edu.washington.cs.knowitall.util.Range;

public class NpList implements ChunkedSource {

    private ChunkedSentence sent;
    private List<Range> npChunkRanges;
    private Range range;

    public NpList(ChunkedSentence sent, Collection<Range> ranges) {
        if (ranges.size() < 2) {
            throw new IllegalArgumentException("Must have at least two Nps in a NpList.");
        }
        npChunkRanges = new ArrayList<Range>(ranges);
        Collections.sort(npChunkRanges, Range.getStartComparator());
        this.sent = sent;
        int start = npChunkRanges.get(0).getStart();
        int end = npChunkRanges.get(npChunkRanges.size()-2).getLastIndex() + 1;
        int length = end - start;
        this.range = new Range(start, length);
    }

    public int getNumNps() {
        return npChunkRanges.size();
    }

    public Range getRange() {
        return range;
    }

    public Iterable<Range> getNpChunkRanges() {
        return npChunkRanges;
    }

    public Range getNpChunkRange(int i) {
        return npChunkRanges.get(i);
    }

    public String getNpString(int i) {
        return sent.getTokensAsString(getNpChunkRange(i));
    }

    public Iterable<String> getNpStrings() {
        ArrayList<String> npStrings = new ArrayList<String>(getNumNps());
        for (int i = 0; i < getNumNps(); i++) {
            npStrings.add(getNpString(i));
        }
        return npStrings;
    }

    public String toString() {
        return "NpList(" + StringUtils.join(getNpStrings().iterator(), ", ") + ")";
    }

    @Override
    public ChunkedSentence getSentence() {
        return sent;
    }

}
