package edu.washington.cs.knowitall.nlp;

import java.util.List;

import edu.washington.cs.knowitall.util.Pair;
import edu.washington.cs.knowitall.util.Range;

/**
 * An interface for objects that have part of speech tags.
 * @author afader
 *
 */
public interface PosTagged extends Tokenized {
    /**
     * @return an iterable object over the POS tags.
     */
    public List<String> getPosTags();

    /**
     * @param start
     * @param length
     * @return an iterable over <code>length</code> POS tags starting at <code>start</code> 
     */
    public List<String> getPosTags(int start, int length);

    /**
     * @param range
     * @return an iterable over the POS tags at the indexes given by <code>range</code>.
     */
    public List<String> getPosTags(Range range);

    /**
     * @param i
     * @return a (token, POS tag) pair at the given index <code>i</code>.
     */
    public Pair<String, String> getPair(int i);
    
    /**
     * @param i
     * @return the POS tag at index <code>i</code>
     */
    public String getPosTag(int i);
}
