package edu.washington.cs.knowitall.nlp.extraction;

import edu.washington.cs.knowitall.nlp.ChunkedSentence;
import edu.washington.cs.knowitall.nlp.OrdinalNumber;
import edu.washington.cs.knowitall.util.Range;

public class OrdinalPhraseExtraction extends ChunkedExtraction {

    private Range npRange;
    private Range nonOrdRange;
    private OrdinalNumber ordinal;

    public OrdinalPhraseExtraction(ChunkedSentence sent, Range npRange, OrdinalNumber ordinal) {
        super(sent, npRange);
        this.npRange = npRange;
        this.ordinal = ordinal;
        computeNonOrdRange();
    }

    public OrdinalPhraseExtraction(ChunkedSentence sent, Range npRange) {
        super(sent, npRange);
        this.npRange = npRange;
        this.ordinal = null;
        computeNonOrdRange();
    }

    public boolean hasOrdinal() {
        return ordinal != null;
    }

    private void computeNonOrdRange() {
        int start;
        if (hasOrdinal()) {
            start = ordinal.getRange().getLastIndex() + 1;
        } else {
            ChunkedSentence sent = getSentence();
            int npStart = npRange.getStart();
            if (sent.getPosTags().get(npStart).equals("DT")) {
                start = npStart + 1;
            } else {
                start = npStart;
            }
        }
        int length = npRange.getLastIndex() + 1 - start;
        nonOrdRange = new Range(start, length);
    }

    public OrdinalNumber getOrdinal() {
        return ordinal;
    }

    public int getOrdinalValue() {
        if (hasOrdinal()) {
            return ordinal.getValue();
        } else {
            return 1;
        }
    }

    public Range getNonOrdinalRange() {
        return nonOrdRange;
    }

    public String getOrdinalString() {
        if (hasOrdinal()) {
            return ordinal.toString();
        } else {
            return "-";
        }
    }

    public String getNonOrdinalString() {
        return getSentence().getTokensAsString(nonOrdRange);
    }

    public String toString() {
        String ordString = getOrdinalString();
        String nonOrdString = getNonOrdinalString();
        return "(" + ordString + ", " + getOrdinalValue() + ", " + nonOrdString + ")";
    }

}
