package edu.washington.cs.knowitall.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RepeatedPatternizedMatcher implements PatternMatcher {

    private String patternStr;
    private int minRepeats = 1;
    private int maxRepeats = Integer.MAX_VALUE;

    public RepeatedPatternizedMatcher(String patternStr) {
        this.patternStr = patternStr;

        // Try compiling, throws an exception if it doesn't work.
        getPattern(1);
    }

    public String getPatternString() {
        return patternStr;
    }

    public void setMinRepeats(int minRepeats) {
        this.minRepeats = minRepeats;
    }

    public void setMaxRepeats(int maxRepeats) {
        this.maxRepeats = maxRepeats;
    }

    public int getMaxRepeats() {
        return maxRepeats;
    }

    public int getMinRepeats() {
        return minRepeats;
    }

    public Pattern getPattern(int repeats) {
        String withRepeats = "(" + getPatternString() + "){" + repeats + "," + repeats + "}";
        return Pattern.compile(withRepeats);
    }

    public Collection<Range> getMatches(String patternized) {
        List<Range> results = new ArrayList<Range>();
        for (int repeats = minRepeats; repeats <= maxRepeats; repeats++) {
            Pattern pattern = getPattern(repeats);
            Matcher matcher = pattern.matcher(patternized);
            int numFound = 0;
            while (matcher.find()) {
                int start = matcher.start();
                int length = matcher.end() - start;
                if (length > 0) {
                    Range range = new Range(start, length);
                    results.add(range);
                    numFound++;
                }     
            }
            if (numFound == 0) {
                break;
            }
        }
        return results;
    }

}
