package edu.washington.cs.knowitall.util;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Scanner;

import edu.washington.cs.knowitall.nlp.PosTagged;


public class TableBasedSentencePatternizer extends PosSentencePatternizer {

    private HashMap<String, Character> tokenTable;
    private HashMap<String, Character> posTagTable;
    private static final Character unknown = '?';

    public static HashMap<String, Character> readTable(File file) throws IOException {
        HashMap<String, Character> table = new HashMap<String, Character>();
        Scanner sc = new Scanner(file);
        while (sc.hasNext()) {
            String line = sc.nextLine();
            Scanner lineScanner = new Scanner(line);
            lineScanner.useDelimiter("\\s");
            String key = lineScanner.next();
            char val = lineScanner.next().charAt(0);
            table.put(key, val);
        }
        return table;
    }

    public TableBasedSentencePatternizer() {
        tokenTable = new HashMap<String, Character>();
        posTagTable = new HashMap<String, Character>();
    }

    public TableBasedSentencePatternizer(HashMap<String, Character> tokenTable, HashMap<String, Character> posTagTable) {
        this.tokenTable = tokenTable;
        this.posTagTable = posTagTable;
    }

    public TableBasedSentencePatternizer(File tokenTableFile, File posTagTableFile) throws IOException {
        tokenTable = readTable(tokenTableFile);
        posTagTable = readTable(posTagTableFile);
    }

    public void setTokenTable(File file) throws IOException {
        tokenTable = readTable(file);
    }

    public void setPosTagTable(File file) throws IOException {
        posTagTable = readTable(file);
    }

    public void setTokenTable(HashMap<String, Character> table) {
        tokenTable = table;
    }

    public void setPosTagTable(HashMap<String, Character> table) {
        posTagTable = table;
    }

    @Override
    public char getPatternChar(PosTagged sentence, int i) {
        String token = sentence.getTokens().get(i);
        String posTag = sentence.getPosTags().get(i);
        if (tokenTable.containsKey(token)) {
            return tokenTable.get(token);
        } else if (posTagTable.containsKey(posTag)) {
            return posTagTable.get(posTag);
        } else {
            return unknown;
        }
    }

}
