package test.edu.washington.cs.knowitall.extractor;


import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

import com.google.common.collect.Iterables;

import edu.washington.cs.knowitall.extractor.RegexRelationExtractor;
import edu.washington.cs.knowitall.nlp.ChunkedSentence;
import edu.washington.cs.knowitall.nlp.extraction.ChunkedExtraction;

public class RegexRelationExtractorTest {

    private ChunkedSentence sent1;
    private String verb = "[POS_VB POS_VBD POS_VBG POS_VBN POS_VBP POS_VBZ]";
    private String notVerbPrep = "[^POS_VB POS_VBD POS_VBG POS_VBN POS_VBP POS_VBZ POS_IN POS_TO]";
    private String prep = "[POS_IN POS_TO]";


    @Before
    public void setUp() throws Exception {
        sent1 = new ChunkedSentence(
                new String[] { "Obama", "was", "a", "professor", "of", "Law", "at", "UChicago", "." },
                new String[] { "NNP", "VBD", "DT", "NN", "IN", "NN", "IN", "NNP", "." },
                new String[] { "B-NP", "O", "B-NP", "I-NP", "I-NP", "I-NP", "O", "B-NP", "O" }
        );
    }

    @Test
    public void testExtract1() {
        String pattern = "(" + verb + notVerbPrep + "*" + prep + "|" + verb + ")+";
        RegexRelationExtractor extractor = new RegexRelationExtractor(pattern);
        Iterable<ChunkedExtraction> extrIter = extractor.extract(sent1);
        List<ChunkedExtraction> extrs = new ArrayList<ChunkedExtraction>();
        Iterables.addAll(extrs, extrIter);
        Assert.assertEquals(1, extrs.size());

        ChunkedExtraction extr = extrs.get(0);
        Assert.assertEquals("was a professor of", extr.toString());
    }

}
